@@ -24,7 +24,7 @@ int handle_reply(xmpp_conn_t * const conn,
 		 void * const userdata)
 {
     xmpp_stanza_t *query, *item;
-    char *type;
+    const char *type;
 
     type = xmpp_stanza_get_type(stanza);
     if (strcmp(type, "error") == 0)
@@ -59,7 +59,7 @@ void conn_handler(xmpp_conn_t * const conn, const xmpp_conn_event_t status,
 	xmpp_stanza_set_name(iq, "iq");
 	xmpp_stanza_set_type(iq, "get");
 	xmpp_stanza_set_id(iq, "active1");
-	xmpp_stanza_set_attribute(iq, "to", "xxxxxxxxx.com");
+	xmpp_stanza_set_to(iq, "xxxxxxxxx.com");
 
 	query = xmpp_stanza_new(ctx);
 	xmpp_stanza_set_name(query, "query");
@@ -104,6 +104,13 @@ int main(int argc, char **argv)
     /* create a connection */
     conn = xmpp_conn_new(ctx);
 
+    /*
+     * also you can disable TLS support or force legacy SSL
+     * connection without STARTTLS
+     *
+     * see xmpp_conn_set_flags() or examples/basic.c
+     */
+
     /* setup authentication information */
     xmpp_conn_set_jid(conn, argv[1]);
     xmpp_conn_set_pass(conn, argv[2]);
