@@ -27,6 +27,8 @@
 #include "hash.h"
 #include "util.h"
 #include "parser.h"
+#include "rand.h"
+#include "snprintf.h"
 
 /** run-time context **/
 
@@ -45,6 +47,7 @@ struct _xmpp_ctx_t {
     const xmpp_mem_t *mem;
     const xmpp_log_t *log;
 
+    xmpp_rand_t *rand;
     xmpp_loop_status_t loop_status;
     xmpp_connlist_t *connlist;
 };
@@ -80,17 +83,6 @@ void xmpp_debug(const xmpp_ctx_t * const ctx,
 		const char * const fmt,
 		...);
 
-/** jid */
-/* these return new strings that must be xmpp_free()'d */
-char *xmpp_jid_new(xmpp_ctx_t *ctx, const char *node,
-                                    const char *domain,
-                                    const char *resource);
-char *xmpp_jid_bare(xmpp_ctx_t *ctx, const char *jid);
-char *xmpp_jid_node(xmpp_ctx_t *ctx, const char *jid);
-char *xmpp_jid_domain(xmpp_ctx_t *ctx, const char *jid);
-char *xmpp_jid_resource(xmpp_ctx_t *ctx, const char *jid);
-
-
 /** connection **/
 
 /* opaque connection object */
@@ -139,10 +131,18 @@ struct _xmpp_handlist_t {
     };
 };
 
-#define SASL_MASK_PLAIN 0x01
-#define SASL_MASK_DIGESTMD5 0x02
-#define SASL_MASK_ANONYMOUS 0x04
-#define SASL_MASK_SCRAMSHA1 0x08
+#define MAX_DOMAIN_LEN 256
+
+#define SASL_MASK_PLAIN     (1 << 0)
+#define SASL_MASK_DIGESTMD5 (1 << 1)
+#define SASL_MASK_ANONYMOUS (1 << 2)
+#define SASL_MASK_SCRAMSHA1 (1 << 3)
+
+enum {
+    XMPP_PORT_CLIENT = 5222,
+    XMPP_PORT_CLIENT_LEGACY_SSL = 5223,
+    XMPP_PORT_COMPONENT = 5347,
+};
 
 typedef void (*xmpp_open_handler)(xmpp_conn_t * const conn);
 
@@ -150,16 +150,22 @@ struct _xmpp_conn_t {
     unsigned int ref;
     xmpp_ctx_t *ctx;
     xmpp_conn_type_t type;
+    int is_raw;
 
     xmpp_conn_state_t state;
     uint64_t timeout_stamp;
     int error;
     xmpp_stream_error_t *stream_error;
-    sock_t sock;
-    tls_t *tls;
 
+    sock_t sock;
+    int ka_timeout; /* TCP keepalive timeout */
+    int ka_interval; /* TCP keepalive interval */
+
+    tls_t *tls;
     int tls_support;
     int tls_disabled;
+    int tls_mandatory;
+    int tls_legacy_ssl;
     int tls_failed; /* set when tls fails, so we don't try again */
     int sasl_support; /* if true, field is a bitfield of supported 
 			 mechanisms */ 
@@ -171,8 +177,6 @@ struct _xmpp_conn_t {
 
     char *lang;
     char *domain;
-    char *connectdomain;
-    char *connectport;
     char *jid;
     char *pass;
     char *bound_jid;
@@ -212,7 +216,9 @@ struct _xmpp_conn_t {
 
 void conn_disconnect(xmpp_conn_t * const conn);
 void conn_disconnect_clean(xmpp_conn_t * const conn);
+void conn_established(xmpp_conn_t * const conn);
 void conn_open_stream(xmpp_conn_t * const conn);
+int conn_tls_start(xmpp_conn_t * const conn);
 void conn_prepare_reset(xmpp_conn_t * const conn, xmpp_open_handler handler);
 void conn_parser_reset(xmpp_conn_t * const conn);
 
@@ -265,9 +271,7 @@ void disconnect_mem_error(xmpp_conn_t * const conn);
 /* auth functions */
 void auth_handle_open(xmpp_conn_t * const conn);
 void auth_handle_component_open(xmpp_conn_t * const conn);
-
-/* replacement snprintf and vsnprintf */
-int xmpp_snprintf (char *str, size_t count, const char *fmt, ...);
-int xmpp_vsnprintf (char *str, size_t count, const char *fmt, va_list arg);
+void auth_handle_open_raw(xmpp_conn_t * const conn);
+void auth_handle_open_stub(xmpp_conn_t * const conn);
 
 #endif /* __LIBSTROPHE_COMMON_H__ */
